// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <memory>

#include "Program.h"
#include "SolverEngine.h"

// Solver implementations in day order.
#include "HistorianHysteria.h"
#include "RedNosedReports.h"
#include "MullItOver.h"
#include "CeresSearch.h"

void Program::run()
{
    std::cout << "### Advent of Code 2024 ###\n";
    runSolvers();
}

void Program::runSolvers()
{
    SolverEngine solverEngine{ getInputPaths() };
    solverEngine.run(*std::make_unique<HistorianHysteria>());
    solverEngine.run(*std::make_unique<RedNosedReports>());
    solverEngine.run(*std::make_unique<MullItOver>());
    solverEngine.run(*std::make_unique<CeresSearch>());
}

std::vector<std::string> Program::getInputPaths() const
{
    return std::vector<std::string>{ "data", "../../data" };
}
