// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include "StringStateMachine.h"

StringStateMachine::StringStateMachine(const std::string& line, StringState& entryState)
{
    line_ = line;
    entryState_ = &entryState;
    currentState_ = entryState_;
    current_ = ' ';
}

void StringStateMachine::run()
{
    currentState_ = entryState_;
    for (auto c : line_)
    {
        current_ = c;
        currentState_->next(this);
    }
}

char StringStateMachine::getCurrent() const
{
    return current_;
}

void StringStateMachine::setState(StringState& state)
{
    currentState_->exit(this);
    currentState_ = &state;
    currentState_->enter(this);
}
