// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include "pch.h"
#include "CppUnitTest.h"

#include "Part1TestContext.h"
#include "Part2TestContext.h"
#include "../Solver.h"
#include "../SolverEngine.h"

#include "../HistorianHysteria.h"

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace UnitTests
{
    TEST_CLASS(HistorianHysteriaTests)
    {
    public:
        TEST_METHOD(FullData1)
        {
            runTest(*std::make_unique<HistorianHysteria>(), 2176849, part1TestContext_);
        }

        TEST_METHOD(FullData2)
        {
            runTest(*std::make_unique<HistorianHysteria>(), 23384288, part2TestContext_);
        }

        TEST_METHOD(ExampleData1)
        {
            runTest(*std::make_unique<HistorianHysteria>(), 11, part1ExampleTestContext_);
        }

        TEST_METHOD(ExampleData2)
        {
            runTest(*std::make_unique<HistorianHysteria>(), 31, part2ExampleTestContext_);
        }

    private:
        Part1TestContext part1TestContext_{ getInputPaths() };
        Part2TestContext part2TestContext_{ getInputPaths() };
        Part1TestContext part1ExampleTestContext_{ getExampleInputPaths() };
        Part2TestContext part2ExampleTestContext_{ getExampleInputPaths() };

        void runTest(Solver& solver, const long long int expected, const TestContext& context)
        {
            SolverEngine solverEngine{ context.getInputPaths() };
            solverEngine.run(solver);
            assertAreEqual(expected, context.getResult(solver));
        }

        std::vector<std::string> getInputPaths() const
        {
            return std::vector<std::string>{ "../../data", "../../../../data" };
        }

        std::vector<std::string> getExampleInputPaths() const
        {
            return std::vector<std::string>{ "../../data/examples", "../../../../data/examples" };
        }

        void assertAreEqual(const long long int expected, const long long int actual) const
        {
            std::wstring message = L"Expected: <" + std::to_wstring(expected) + L"> Actual: <" + std::to_wstring(actual) + L">";
            Assert::IsTrue(expected == actual, message.c_str());
        }
    };
}
